/**********************************************************
   browseled.c
   Rabbit Semiconductor, 2006

	This program is used with RCM4000 series controllers
	with prototyping boards.

	Description
	===========
	This program demonstrates a basic controller running a
	WEB page. Two "device LED's" are created with two
	buttons to toggle them.  Users can browse to the device
	and change the status of the lights. The LED's on the
	prototyping board will match the ones on the web page.

   This program is adapted from \Samples\TCPIP\ssi.c.

	Instructions
	============
	1. Make changes below in the configuration section to
	   match your application.
	2. Compile and run this program.
	3. With your WEB browser access the WEB page running on
	   the controller.
	4. View LEDS on Web page and the prototyping board, DS2
   	   and DS3, to see that they match-up when changing them
	   via the WEB page control button.

**********************************************************/

#class auto
#use rcm40xx.lib

#define S2  4
#define S3  5
#define DS2 2
#define DS3 3
#define ON 0
#define OFF 1

/***********************************
 * Configuration Section           *
 * ---------------------           *
 * All fields in this section must *
 * be altered to match your local  *
 * network settings.               *
 ***********************************/

#define DEFAULT_IP	"192.168.1.200"
#define DEFAULT_MASK	"255.255.255.0"
#define DEFAULT_GATE	"192.168.1.1"
#define DEFAULT_GIP	"0.0.0.0"
#define DEFAULT_PORT	80

/*
 * Pick the predefined TCP/IP configuration for this sample.  See
 * LIB\TCPIP\TCP_CONFIG.LIB for instructions on how to set the
 * configuration.
 */
#define TCPCONFIG		1


/*
 * TCP/IP modification - reduce TCP socket buffer
 * size, to allow more connections. This can be increased,
 * with increased performance, if the number of sockets
 * are reduced.  Note that this buffer size is split in
 * two for TCP sockets--1024 bytes for send and 1024 bytes
 * for receive.
 */
#define TCP_BUF_SIZE	2048

/*
 * Web server configuration
 */

/*
 * Define the number of HTTP servers and socket buffers.
 * With tcp_reserveport(), fewer HTTP servers are needed.
 */
#define HTTP_MAXSERVERS 1
#define MAX_TCP_SOCKET_BUFFERS 1

/*
 * Our web server as seen from the clients.
 * This should be the address that the clients (netscape/IE)
 * use to access your server. Usually, this is your IP address.
 * If you are behind a firewall, though, it might be a port on
 * the proxy, that will be forwarded to the Rabbit board. The
 * commented out line is an example of such a situation.
 */

/********************************
 * End of configuration section *
 ********************************/

#memmap xmem
#use "dcrtcp.lib"
#use "http.lib"

/*
 *  Notice that we have ximported in the source code for
 *  this program.  This allows us to <!--#include file="browseled.c"-->
 *  in the pages/showsrc.shtml.
 *
 */

#ximport "/HOME/WEBSAMPLE/pages/browseled.shtml"	index_shtml
#ximport "/HOME/WEBSAMPLE/pages/setparam.shtml" 	setparam_shtml
#ximport "/HOME/WEBSAMPLE/pages/rabbit1.gif"    	rabbit1_gif
#ximport "/HOME/WEBSAMPLE/pages/ledon.gif"      	ledon_gif
#ximport "/HOME/WEBSAMPLE/pages/ledoff.gif"     	ledoff_gif
#ximport "/HOME/WEBSAMPLE/pages/button.gif"     	button_gif
#ximport "/HOME/WEBSAMPLE/pages/showsrc.shtml"  	showsrc_shtml
#ximport "/HOME/WEBSAMPLE/browseled.c"          	browseled_c


/*
 *  In this case the .html is not the first type in the
 *  type table.  This causes the default (no extension)
 *  to assume the shtml_handler.
 *
 */

/* the default for / must be first */
SSPEC_MIMETABLE_START
   SSPEC_MIME_FUNC( ".shtml", "text/html", shtml_handler),	// ssi
   SSPEC_MIME( ".html", "text/html"),		// html
   SSPEC_MIME( ".cgi", ""),					// cgi
   SSPEC_MIME( ".gif", "image/gif")
SSPEC_MIMETABLE_END

//		Variables to save settings
static long	ipaddress;
static long	netmask;
static long	gateaddress;
static long	glbipaddr;
static unsigned int httpport;

//		Global variables
char	modeflag;
long	netmask_tmp;
long	glbipaddr_tmp;
unsigned int httpport_tmp;

#define MAX_FORMSIZE	32
typedef struct {
	char *name;
	char value[MAX_FORMSIZE];
} FORMType;
FORMType FORMSpec[5];

//		For ssi
char	myipurl[22];
char	yourip[16];
/*
 *  Each ledx contains a text string that is either
 *  "ledon.gif" or "ledoff.gif"  This string is toggled
 *  each time the ledxtoggle.cgi is requested from the
 *  browser.
 *
 */
char	led1[15];
char	led2[15];
//		For "setparam.shtml" page
char	curipaddr[16];
char	curmask[16];
char	curgate[16];
char	curglbip[16];
char	curport[6];
char	okmsg[41];

/*
 * parse the url-encoded POST data into the FORMSpec struct
 * (ie: parse 'foo=bar&baz=qux' into the struct
 */
int parse_post(HttpState* state) {
	int	retval;
	int	i;

	// state->s is the socket structure, and state->p is pointer
	// into the HTTP state buffer (initially pointing to the beginning
	// of the buffer).  Note that state->p was set up in the submit
	// CGI function.  Also note that we read up to the content_length,
	// or HTTP_MAXBUFFER, whichever is smaller.  Larger POSTs will be
	// truncated.
	retval = sock_aread(&state->s, state->p,
			(state->content_length < HTTP_MAXBUFFER-1)?
			(int)state->content_length:HTTP_MAXBUFFER-1);
	if (retval < 0) {
		// Error--just bail out
		return 1;
	}

	// Using the subsubstate to keep track of how much data we have received
	state->subsubstate += retval;

	if (state->subsubstate >= state->content_length) {
		// NULL-terminate the content buffer
		state->buffer[(int)state->content_length] = '\0';

		// Scan the received POST information into the FORMSpec structure
		for(i=0; i<(sizeof(FORMSpec)/sizeof(FORMType)); i++) {
			http_scanpost(FORMSpec[i].name, state->buffer, FORMSpec[i].value,
				MAX_FORMSIZE);
		}

		// Finished processing--returning 1 indicates that we are done
		return 1;
	}
	// Processing not finished--return 0 so that we can be called again
	return 0;
}

/*
 *  Instead of sending other text back from the cgi's
 *  we have decided to redirect them to the original page.
 *  the cgi_redirectto forms a header which will redirect
 *  the browser back to the main page.
 *
 *  This procedure, which returns my URL, is executed
 *  every time when main page is loaded. This demonstrates
 *  capability of dynamically changing redirection.
 */

char *myurl(HttpState* state) {
	static char	URL[64];
	sockaddr	peeraddr;
	sockaddr	svraddr;
	char		tmpstr[32];

	getpeername((sock_type*) http_getSocket(state), &peeraddr, NULL);
	getsockname((sock_type*) http_getSocket(state), &svraddr, NULL);

	if((peeraddr.s_ip & netmask_tmp) == (svraddr.s_ip & netmask_tmp) )
		inet_ntoa(tmpstr, svraddr.s_ip);
	else
		inet_ntoa(tmpstr, glbipaddr_tmp);

	sprintf(URL, "http://%s:%ld/", tmpstr,(unsigned long) httpport_tmp);

	if(modeflag) 	strcat(URL, "setparam.shtml");

   return URL;
}

int dispmainp(HttpState* state) {
	sockaddr	peeraddr;
	sockaddr	svraddr;
	char		tmpstr[32];

	getpeername((sock_type*) http_getSocket(state), &peeraddr, NULL);
	getsockname((sock_type*) http_getSocket(state), &svraddr, NULL);

	strcpy(yourip, inet_ntoa(tmpstr, peeraddr.s_ip));

	if((peeraddr.s_ip & netmask_tmp) == (svraddr.s_ip & netmask_tmp) )
		inet_ntoa(tmpstr, svraddr.s_ip);
	else
		inet_ntoa(tmpstr, glbipaddr_tmp);

	sprintf(myipurl,"%s:%ld",tmpstr,(unsigned long) httpport_tmp);

	cgi_continue(state,"mainp.shtml");
	return 0;
}

/*
 * Sample submit.cgi function
 */
int submit(HttpState* state) {
	char		tmpstr[32];

	switch(state->substate) {
		case 0:
			/* init the FORMSpec data */
			FORMSpec[0].value[0] = '\0';
			FORMSpec[1].value[0] = '\0';
			FORMSpec[2].value[0] = '\0';
			FORMSpec[3].value[0] = '\0';
			FORMSpec[4].value[0] = '\0';

			state->p = state->buffer;
			state->substate++;
			break;

		case 1:
			/* parse the POST information */
			if(parse_post(state)) {
				if(modeflag) {
            	ipaddress = aton(FORMSpec[0].value);
				   netmask = aton(FORMSpec[1].value);
				   gateaddress = aton(FORMSpec[2].value);
				   glbipaddr = aton(FORMSpec[3].value);
				   httpport = (unsigned int) atol(FORMSpec[4].value);

				   strcpy(curipaddr,inet_ntoa(tmpstr, ipaddress));
				   strcpy(curmask,inet_ntoa(tmpstr, netmask));
				   strcpy(curgate,inet_ntoa(tmpstr, gateaddress));
				   strcpy(curglbip,inet_ntoa(tmpstr, glbipaddr));
				   strcpy(curport,ltoa((unsigned long) httpport, tmpstr));

				   strcpy(okmsg,"ZbgƁAύXLɂȂ܂B");
         	} else
				   strcpy(okmsg,"ݒ͂ł܂B");

				cgi_redirectto(state,"setparam.shtml");

				return 0;
			}
			break;

		default:
			state->substate = 0;
			return 1;
	}

	return 0;
}

/*
 *  REDIRECTTO is used by each ledxtoggle cgi's to tell the
 *  browser which page to hit next.  The default REDIRECTTO
 *  assumes that you are serving a page that does not have
 *  any address translation applied to it.
 *  => Modified to accept an address translation applied by
 *     a router.
 */

 int led1toggle(HttpState* state) {
	if(! modeflag) {
		if (strcmp(led1,"ledon.gif")==0)
			strcpy(led1,"ledoff.gif");
      else
			strcpy(led1,"ledon.gif");
	}
	cgi_redirectto(state,myurl(state));
	return 0;
}

int led2toggle(HttpState* state) {
	if(! modeflag) {
		if (strcmp(led2,"ledon.gif")==0)
			strcpy(led2,"ledoff.gif");
		else
			strcpy(led2,"ledon.gif");
	}
	cgi_redirectto(state,myurl(state));
	return 0;
}


SSPEC_RESOURCETABLE_START
   SSPEC_RESOURCE_XMEMFILE("/mainp.shtml", index_shtml),
   SSPEC_RESOURCE_XMEMFILE("/showsrc.shtml", showsrc_shtml),
   SSPEC_RESOURCE_XMEMFILE("/setparam.shtml", setparam_shtml),
   SSPEC_RESOURCE_XMEMFILE("/rabbit1.gif", rabbit1_gif),
   SSPEC_RESOURCE_XMEMFILE("/ledon.gif", ledon_gif),
   SSPEC_RESOURCE_XMEMFILE("/ledoff.gif", ledoff_gif),
   SSPEC_RESOURCE_XMEMFILE("/button.gif", button_gif),

   SSPEC_RESOURCE_XMEMFILE("browseled.c", browseled_c),

   SSPEC_RESOURCE_ROOTVAR("myipurl", myipurl, PTR16, "%s"),
   SSPEC_RESOURCE_ROOTVAR("yourip", yourip, PTR16, "%s"),
   SSPEC_RESOURCE_ROOTVAR("led1", led1, PTR16, "%s"),
   SSPEC_RESOURCE_ROOTVAR("led2", led2, PTR16, "%s"),
   SSPEC_RESOURCE_ROOTVAR("curipaddr", curipaddr, PTR16, "%s"),
   SSPEC_RESOURCE_ROOTVAR("curmask", curmask, PTR16, "%s"),
   SSPEC_RESOURCE_ROOTVAR("curgate", curgate, PTR16, "%s"),
   SSPEC_RESOURCE_ROOTVAR("curglbip", curglbip, PTR16, "%s"),
   SSPEC_RESOURCE_ROOTVAR("curport", curport, PTR16, "%s"),
   SSPEC_RESOURCE_ROOTVAR("okmsg", okmsg, PTR16, "%s"),

   SSPEC_RESOURCE_FUNCTION("/", dispmainp),
   SSPEC_RESOURCE_FUNCTION("/index.html", dispmainp),
   SSPEC_RESOURCE_FUNCTION("/index.shtml", dispmainp),
   SSPEC_RESOURCE_FUNCTION("/led1tog.cgi", led1toggle),
   SSPEC_RESOURCE_FUNCTION("/led2tog.cgi", led2toggle),
   SSPEC_RESOURCE_FUNCTION("/submit.cgi", submit)
SSPEC_RESOURCETABLE_END


void update_outputs() {
	/* update O0 */
	if (strcmp(led1,"ledon.gif"))
		BitWrPortI(PBDR, &PBDRShadow, OFF, DS2);
	else
		BitWrPortI(PBDR, &PBDRShadow, ON, DS2);

	/* update O1 */
	if (strcmp(led2,"ledon.gif"))
		BitWrPortI(PBDR, &PBDRShadow, OFF, DS3);
	else
		BitWrPortI(PBDR, &PBDRShadow, ON, DS3);

}

/*
 * Check the status of switch 1
 */
cofunc void CheckSwitch1() {
	if (BitRdPortI(PBDR, S2))			// wait for switch press
		abort;					// if button not down skip out
	waitfor(DelayMs(50));				// wait 50 ms
	if (BitRdPortI(PBDR, S2))			// wait for switch press
		abort;					// if button not still down exit

	if (strcmp(led1,"ledon.gif")==0)
		strcpy(led1,"ledoff.gif");
	else
		strcpy(led1,"ledon.gif");

	while (1) {
		waitfor(BitRdPortI(PBDR, S2));		// wait for button to go up
		waitfor(DelayMs(200));			// wait additional 200 ms
		if (BitRdPortI(PBDR, S2))		// wait for switch press
			break;				// if button still up break out of while loop
	}
}

/*
 * Check the status of switch 2
 */
cofunc void CheckSwitch2() {
	if (BitRdPortI(PBDR, S3))			// wait for switch press
		abort;					// if button not down skip out
	waitfor(DelayMs(50));				// wait 50 ms
	if (BitRdPortI(PBDR, S3))			// wait for switch press
		abort;					// if button not still down exit

	if (strcmp(led2,"ledon.gif")==0)
		strcpy(led2,"ledoff.gif");
	else
		strcpy(led2,"ledon.gif");

	while (1) {
		waitfor(BitRdPortI(PBDR, S3));		// wait for button to go up
		waitfor(DelayMs(200));			// wait additional 200 ms
		if (BitRdPortI(PBDR, S3))		// wait for switch press
			break;				// if button still up break out of while loop
	}
}


/*
 *
 *		Check if any of the switches have been pressed
 *    if so then alternate LED status.
 *
 */
void check_switch_status(void) {
	costate {
		waitfordone { CheckSwitch1(); }
	}
	costate {
		waitfordone { CheckSwitch2(); }
	}
}

void check_init_switch(void)
{
	char		tmpstr[32];

	if (! BitRdPortI(PBDR, S2)) {			// if switch pressed
		while(! BitRdPortI(PBDR, S2));		// wait for button to go up

		ipaddress = aton(DEFAULT_IP);
		netmask = aton(DEFAULT_MASK);
		gateaddress = aton(DEFAULT_GATE);
		glbipaddr = aton(DEFAULT_GIP);
		httpport = DEFAULT_PORT;
	}

	modeflag = 0;
	if (! BitRdPortI(PBDR, S3)) {			// if switch pressed
		while(! BitRdPortI(PBDR, S3));		// wait for button to go up

		modeflag = 1;
		strcpy(curipaddr,inet_ntoa(tmpstr, ipaddress));
		strcpy(curmask,inet_ntoa(tmpstr, netmask));
		strcpy(curgate,inet_ntoa(tmpstr, gateaddress));
		strcpy(curglbip,inet_ntoa(tmpstr, glbipaddr));
		strcpy(curport,ltoa((unsigned long) httpport, tmpstr));

		FORMSpec[0].name = "newipaddr";
		FORMSpec[1].name = "newmask";
		FORMSpec[2].name = "newgate";
		FORMSpec[3].name = "newglbip";
		FORMSpec[4].name = "newport";
	}

	ifconfig(IF_DEFAULT, IFS_DOWN, IFS_IPADDR, ipaddress,
   		IFS_NETMASK, netmask, IFS_ROUTER_SET, gateaddress, IFS_UP, IFS_END);

	httpport_tmp = httpport;
	netmask_tmp = netmask;
	glbipaddr_tmp = glbipaddr;
}

main() {

	brdInit();					//initialize board for this demo

	sock_init();
	check_init_switch();

	strcpy(led1,"ledon.gif");
	strcpy(led2,"ledoff.gif");

	okmsg[0] = '\0';

	http_init();
	tcp_reserveport(httpport_tmp);

	while (1) {
		check_switch_status();
		update_outputs();
		http_handler(httpport_tmp);		//http_handler (in HTTP.LIB) must be modified
	}
}